/*******************************************************************************
*    MultiButton 1.0.                                                          *
*    Verwaltung eines Domotikasystem mit O2-Joggler                            *
*    nach einer Idee von Leonardo Buffa <leonardo.buffa@gmail.com>             *
*    Copyright (C) 2013-averyfarwaydate Luca Bertoncello                       *
*    Hartigstrasse, 12 - 01127 Dresden Deutschland                             *
*    E-Mail: lucabert@lucabert.de, lucabert@lucabert.com                       *
*    http://www.lucabert.de/  http://www.lucabert.com/                         *
*                                                                              *
*    This program is free software; you can redistribute it and/or modify      *
*    it under the terms of the GNU General Public License as published by      *
*    the Free Software Foundation; version 2 of the License.                   *
*                                                                              *
*    This program is distributed in the hope that it will be useful,           *
*    but WITHOUT ANY WARRANTY; without even the implied warranty of            *
*    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
*    GNU General Public License for more details.                              *
*                                                                              *
*    You should have received a copy of the GNU General Public License         *
*    along with this program; if not, write to the Free Software               *
*    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA *
*******************************************************************************/

#include "CurlReader.h"

/**
 * Callback-Funktion von cURL.
 * Die Daten werden vom Server gelesen
 */
static size_t WriteMemoryCallback(void *contents, size_t size, size_t nmemb, void *userp)
{
  size_t realsize = size * nmemb;

  struct MemoryStruct *mem = (struct MemoryStruct *)userp;

  mem->memory = (char*) realloc(mem->memory, mem->size + realsize + 1);
  if(mem->memory == NULL)
  {
    printf("Not enough memory (realloc returned NULL)\n");
    return 0;
  }

  memcpy(&(mem->memory[mem->size]), contents, realsize);
  mem->size += realsize;
  mem->memory[mem->size] = 0;

  return realsize;
}

/**
 * Konstruktor
 *
 * @param wxString url                    URL herunterzuladen
 */
CurlReader::CurlReader(wxString url)
{
  this->chunk.memory = (char*) malloc(1);
  this->chunk.size = 0;

  curl_global_init(CURL_GLOBAL_ALL);

  this->curlHandle = curl_easy_init();
  curl_easy_setopt(this->curlHandle, CURLOPT_TIMEOUT, 5); // 5 Sekunden Timeout
  curl_easy_setopt(this->curlHandle, CURLOPT_URL, (const char*) url.mb_str());
  curl_easy_setopt(this->curlHandle, CURLOPT_WRITEFUNCTION, WriteMemoryCallback);
  curl_easy_setopt(this->curlHandle, CURLOPT_WRITEDATA, (void *)&this->chunk);
  curl_easy_setopt(this->curlHandle, CURLOPT_USERAGENT, "Multibutton/1.0");
}

CurlReader::~CurlReader()
{
  if(this->chunk.memory)
    free(this->chunk.memory);
}

/**
 * Holt den Inhalt der Datei
 *
 * @return wxInputStream                  Die Datei als wxInputStream
 */
wxInputStream *CurlReader::getFile()
{
  wxMemoryInputStream *content;

  curl_easy_cleanup(this->curlHandle);

  content = new wxMemoryInputStream(this->chunk.memory, this->chunk.size);

  curl_global_cleanup();

  return content;
}

/**
 * Lädt die Datei herunter
 *
 * @return bool                           true wenn alles in Ordnung,
 *                                        false bei Fehler
 */
bool CurlReader::load()
{
  this->res = curl_easy_perform(this->curlHandle);

  if(this->res != CURLE_OK)
    return false;

  return true;
}

/**
 * Gibt die HTTP-Fehler als String zurück
 *
 * @return wxString                       die Fehlermeldung
 */
wxString CurlReader::errors()
{
  return wxString::FromUTF8(curl_easy_strerror(this->res));
}
